/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.arrebol.taxiservicios.model.predio;

import com.arrebol.taxiservicios.model.catalog.Location;
import com.arrebol.taxiservicios.model.core.Person;
import com.arrebol.taxiservicios.model.core.User;
import com.arrebol.taxiservicios.model.enums.CompromisoPago;
import com.arrebol.taxiservicios.model.enums.GenericEnumType;
import com.arrebol.taxiservicios.model.enums.TipoProyecto;
import java.io.Serializable;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import org.hibernate.annotations.GenericGenerator;

/**
 *
 * @author Picasso
 */
@Entity
@Table(name = "proyecto")
public class Predio implements Serializable {

    private static final long serialVersionUID = -6653037938225719593L;

    @Id
    @GeneratedValue(generator = "uuid")
    @GenericGenerator(name = "uuid", strategy = "uuid2")
    @Column(name = "id", length = 36)
    private String id;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(
            name = "id_propietario",
            referencedColumnName = "id",
            nullable = false
    )
    private Person propetiario;

    @Column(name = "nombre_proyecto", nullable = false)
    private String nombreProyecto;

    @Column(name = "costo_terreno", nullable = false)
    private Double costo;

    @Column(name = "anticipo", length = 25, nullable = false)
    private Double anticipo;

    @Enumerated(EnumType.STRING)
    @Column(name = "compromiso_pago", nullable = false)
    private CompromisoPago compromisoPago;

    @Column(name = "comision_mensual", length = 25, nullable = false)
    private Double comisionMensual;

    @Column(name = "interes_mensual", length = 25, nullable = false)
    private Double interesMensual;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(
            name = "id_location",
            referencedColumnName = "id",
            nullable = false
    )
    private Location location;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(
            name = "created_by",
            referencedColumnName = "id",
            nullable = false
    )
    private User createdBy;

    @Temporal(TemporalType.TIMESTAMP)
    @Column(name = "created_on", length = 19)
    private Date createdOn;

    @ManyToOne(fetch = FetchType.LAZY, optional = true)
    @JoinColumn(
            name = "last_updated_by",
            referencedColumnName = "id",
            nullable = true
    )
    private User lastUpdatedBy;

    @Temporal(TemporalType.TIMESTAMP)
    @Column(name = "last_updated_on", length = 19)
    private Date lastUpdatedOn;

    @Enumerated(EnumType.STRING)
    @Column(name = "estatus_activo", nullable = true)
    private GenericEnumType estatusActivo;

    @OneToMany(
            mappedBy = "predio",
            cascade = CascadeType.ALL,
            fetch = FetchType.LAZY,
            orphanRemoval = true
    )
    private List<Manzana> manzanas;

    @Column(name = "mandado")
    private Double mandado;

    @Column(name = "cobranza_atraso")
    private Double cobranzaAtraso;

    @Column(name = "croquis")
    private String croquis;

    @Enumerated(EnumType.STRING)
    @Column(name = "tipo_proyecto", nullable = false)
    private TipoProyecto tipoProyecto;

    @Column(name = "abreviatura")
    private String abreviatura;

    @Column(name = "costo_metro_cuadrado")
    private String costoMetroCuadrado;

    /**
     *
     * @param id
     * @param propetiario
     * @param nombreProyecto
     */
    public Predio(String id, Person propetiario, String nombreProyecto) {
        this.id = id;
        this.propetiario = propetiario;
        this.nombreProyecto = nombreProyecto;
    }

    /**
     *
     * @param id
     * @param propetiario
     * @param nombreProyecto
     * @param manzanas
     */
    public Predio(String id, Person propetiario, String nombreProyecto, List<Manzana> manzanas) {
        this.id = id;
        this.propetiario = propetiario;
        this.nombreProyecto = nombreProyecto;
        this.manzanas = manzanas;
    }

    public Predio() {
    }

    public Predio(String id) {
        this.id = id;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getNombreProyecto() {
        return nombreProyecto;
    }

    public void setNombreProyecto(String nombreProyecto) {
        this.nombreProyecto = nombreProyecto;
    }

    public Person getPropetiario() {
        return propetiario;
    }

    public void setPropetiario(Person propetiario) {
        this.propetiario = propetiario;
    }

    public Double getCosto() {
        return costo;
    }

    public void setCosto(Double costo) {
        this.costo = costo;
    }

    public Double getAnticipo() {
        return anticipo;
    }

    public void setAnticipo(Double anticipo) {
        this.anticipo = anticipo;
    }

    public User getCreatedBy() {
        return createdBy;
    }

    public void setCreatedBy(User createdBy) {
        this.createdBy = createdBy;
    }

    public Location getLocation() {
        return location;
    }

    public void setLocation(Location location) {
        this.location = location;
    }

    public Date getCreatedOn() {
        return createdOn;
    }

    public void setCreatedOn(Date createdOn) {
        this.createdOn = createdOn;
    }

    public User getLastUpdatedBy() {
        return lastUpdatedBy;
    }

    public void setLastUpdatedBy(User lastUpdatedBy) {
        this.lastUpdatedBy = lastUpdatedBy;
    }

    public Date getLastUpdatedOn() {
        return lastUpdatedOn;
    }

    public void setLastUpdatedOn(Date lastUpdatedOn) {
        this.lastUpdatedOn = lastUpdatedOn;
    }

    public CompromisoPago getCompromisoPago() {
        return compromisoPago;
    }

    public void setCompromisoPago(CompromisoPago compromisoPago) {
        this.compromisoPago = compromisoPago;
    }

    public Double getComisionMensual() {
        return comisionMensual;
    }

    public void setComisionMensual(Double comisionMensual) {
        this.comisionMensual = comisionMensual;
    }

    public Double getInteresMensual() {
        return interesMensual;
    }

    public void setInteresMensual(Double interesMensual) {
        this.interesMensual = interesMensual;
    }

    public List<Manzana> getManzanas() {
        return manzanas;
    }

    public void setManzanas(List<Manzana> manzanas) {
        this.manzanas = manzanas;
    }

    public GenericEnumType getEstatusActivo() {
        return estatusActivo;
    }

    public void setEstatusActivo(GenericEnumType estatusActivo) {
        this.estatusActivo = estatusActivo;
    }

    public Double getMandado() {
        return mandado;
    }

    public void setMandado(Double mandado) {
        this.mandado = mandado;
    }

    public Double getCobranzaAtraso() {
        return cobranzaAtraso;
    }

    public void setCobranzaAtraso(Double cobranzaAtraso) {
        this.cobranzaAtraso = cobranzaAtraso;
    }

    public String getCroquis() {
        return croquis;
    }

    public void setCroquis(String croquis) {
        this.croquis = croquis;
    }

    public TipoProyecto getTipoProyecto() {
        return tipoProyecto;
    }

    public void setTipoProyecto(TipoProyecto tipoProyecto) {
        this.tipoProyecto = tipoProyecto;
    }

    @Override
    public String toString() {
        return nombreProyecto + " " + propetiario;
    }

    public String getAbreviatura() {
        return abreviatura;
    }

    public void setAbreviatura(String abreviatura) {
        this.abreviatura = abreviatura;
    }

    public String getCostoMetroCuadrado() {
        return costoMetroCuadrado;
    }

    public void setCostoMetroCuadrado(String costoMetroCuadrado) {
        this.costoMetroCuadrado = costoMetroCuadrado;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final Predio other = (Predio) obj;
        if (!Objects.equals(this.id, other.id)) {
            return false;
        }
        return true;
    }

}
