/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.arrebol.taxiservicios.model.spa;

import com.arrebol.taxiservicios.model.catalog.Location;
import com.arrebol.taxiservicios.model.core.Person;
import com.arrebol.taxiservicios.model.core.User;
import com.arrebol.taxiservicios.model.enums.EstatusVenta;
import com.arrebol.taxiservicios.model.enums.GenericEnumType;
import com.arrebol.taxiservicios.model.enums.PeriodoPago;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;
import org.hibernate.annotations.GenericGenerator;

/**
 *
 * @author Crov
 */
@Entity
@Table(name = "ventas")
public class Ventas implements Serializable {

   private static final long serialVersionUID = -6653037938225719593L;

   @Id
   @GeneratedValue(generator = "uuid")
   @GenericGenerator(name = "uuid", strategy = "uuid2")
   @Column(name = "id", length = 36)
   private String id;

   @ManyToOne(fetch = FetchType.LAZY, optional = false)
   @JoinColumn(
           name = "id_location",
           referencedColumnName = "id",
           nullable = false
   )
   private Location location;

   @ManyToOne(fetch = FetchType.LAZY, optional = true)
   @JoinColumn(
           name = "id_cliente",
           referencedColumnName = "id",
           nullable = true
   )
   private Person cliente;

   @ManyToOne(fetch = FetchType.LAZY, optional = true)
   @JoinColumn(
           name = "id_cabinas",
           referencedColumnName = "id",
           nullable = true
   )
   private Cabinas cabina;

   @ManyToOne(fetch = FetchType.LAZY, optional = true)
   @JoinColumn(
           name = "id_catalogo_productos",
           referencedColumnName = "id",
           nullable = true
   )
   private CatalogoProductos producto;

   @Column(name = "productos_vendidos")
   private String productosVendidos;

   @Enumerated(EnumType.STRING)
   @Column(name = "estatus_activo", nullable = false)
   private GenericEnumType estatusActivo;

   @Enumerated(EnumType.STRING)
   @Column(name = "estatus_venta", nullable = false)
   private EstatusVenta estatusVenta;

   @Column(name = "numero_pagos")
   private Integer numeroPagos;

   @Enumerated(EnumType.STRING)
   @Column(name = "periodo_pago", nullable = true)
   private PeriodoPago periodoPago;

   @Temporal(TemporalType.TIMESTAMP)
   @Column(name = "fecha_venta")
   private Date fechaVenta;

   @Temporal(TemporalType.TIMESTAMP)
   @Column(name = "fecha_programado")
   private Date fechaProgramado;

   @Column(name = "pago")
   private Double pago;

   @Column(name = "anticipo")
   private Double anticipo;

   @Column(name = "total_pago")
   private Double totalPago;

   @Column(name = "cantidad_pagada")
   private Double cantidadPagada;

   @Temporal(TemporalType.TIMESTAMP)
   @Column(name = "fecha_a_pagar")
   private Date fechaAPagar;

   @Enumerated(EnumType.STRING)
   @Column(name = "periodo_cesiones", nullable = true)
   private PeriodoPago periodoCesiones;

   @Column(name = "numero_cesiones")
   private Integer numeroCesiones;

   @Temporal(TemporalType.TIMESTAMP)
   @Column(name = "created_on")
   private Date createdOn;

   @ManyToOne(fetch = FetchType.LAZY, optional = false)
   @JoinColumn(
           name = "created_by",
           referencedColumnName = "id",
           nullable = false
   )
   private User createdBy;

   @Temporal(TemporalType.TIMESTAMP)
   @Column(name = "last_updated_on")
   private Date lastUpdatedOn;

   @ManyToOne(fetch = FetchType.LAZY, optional = true)
   @JoinColumn(
           name = "last_updated_by",
           referencedColumnName = "id",
           nullable = true
   )
   private User lastUpdatedBy;

   @Column(name = "descuento_fijo")
   private Double descuentoFijo;

   @Column(name = "porcentaje_descuento")
   private Double porcentajeDescuento;

   @Column(name = "descuento_porcentaje")
   private Double descuentoPorcentaje;

   @Transient
   private String fechaFormateada;

   public Ventas() {
   }

   public Ventas(String id) {
      this.id = id;
   }

   public String getId() {
      return id;
   }

   public void setId(String id) {
      this.id = id;
   }

   public Cabinas getCabina() {
      return cabina;
   }

   public void setCabina(Cabinas cabina) {
      this.cabina = cabina;
   }

   public CatalogoProductos getProducto() {
      return producto;
   }

   public void setProducto(CatalogoProductos producto) {
      this.producto = producto;
   }

   public GenericEnumType getEstatusActivo() {
      return estatusActivo;
   }

   public void setEstatusActivo(GenericEnumType estatusActivo) {
      this.estatusActivo = estatusActivo;
   }

   public EstatusVenta getEstatusVenta() {
      return estatusVenta;
   }

   public void setEstatusVenta(EstatusVenta estatusVenta) {
      this.estatusVenta = estatusVenta;
   }

   public Integer getNumeroPagos() {
      return numeroPagos;
   }

   public void setNumeroPagos(Integer numeroPagos) {
      this.numeroPagos = numeroPagos;
   }

   public Date getFechaVenta() {
      return fechaVenta;
   }

   public void setFechaVenta(Date fechaVenta) {
      this.fechaVenta = fechaVenta;
   }

   public Date getFechaProgramado() {
      return fechaProgramado;
   }

   public void setFechaProgramado(Date fechaProgramado) {
      this.fechaProgramado = fechaProgramado;
   }

   public Double getTotalPago() {
      return totalPago;
   }

   public void setTotalPago(Double totalPago) {
      this.totalPago = totalPago;
   }

   public Date getCreatedOn() {
      return createdOn;
   }

   public void setCreatedOn(Date createdOn) {
      this.createdOn = createdOn;
   }

   public User getCreatedBy() {
      return createdBy;
   }

   public void setCreatedBy(User createdBy) {
      this.createdBy = createdBy;
   }

   public Date getLastUpdatedOn() {
      return lastUpdatedOn;
   }

   public void setLastUpdatedOn(Date lastUpdatedOn) {
      this.lastUpdatedOn = lastUpdatedOn;
   }

   public User getLastUpdatedBy() {
      return lastUpdatedBy;
   }

   public void setLastUpdatedBy(User lastUpdatedBy) {
      this.lastUpdatedBy = lastUpdatedBy;
   }

   public Double getPago() {
      return pago;
   }

   public void setPago(Double pago) {
      this.pago = pago;
   }

   public Double getAnticipo() {
      return anticipo;
   }

   public void setAnticipo(Double anticipo) {
      this.anticipo = anticipo;
   }

   public Location getLocation() {
      return location;
   }

   public void setLocation(Location location) {
      this.location = location;
   }

   public Person getCliente() {
      return cliente;
   }

   public void setCliente(Person cliente) {
      this.cliente = cliente;
   }

   public PeriodoPago getPeriodoPago() {
      return periodoPago;
   }

   public void setPeriodoPago(PeriodoPago periodoPago) {
      this.periodoPago = periodoPago;
   }

   public Double getCantidadPagada() {
      return cantidadPagada;
   }

   public void setCantidadPagada(Double cantidadPagada) {
      this.cantidadPagada = cantidadPagada;
   }

   public Date getFechaAPagar() {
      return fechaAPagar;
   }

   public void setFechaAPagar(Date fechaAPagar) {
      this.fechaAPagar = fechaAPagar;
   }

   public PeriodoPago getPeriodoCesiones() {
      return periodoCesiones;
   }

   public void setPeriodoCesiones(PeriodoPago periodoCesiones) {
      this.periodoCesiones = periodoCesiones;
   }

   public Integer getNumeroCesiones() {
      return numeroCesiones;
   }

   public void setNumeroCesiones(Integer numeroCesiones) {
      this.numeroCesiones = numeroCesiones;
   }

   public Double getDescuentoFijo() {
      return descuentoFijo;
   }

   public void setDescuentoFijo(Double descuentoFijo) {
      this.descuentoFijo = descuentoFijo;
   }

   public Double getPorcentajeDescuento() {
      return porcentajeDescuento;
   }

   public void setPorcentajeDescuento(Double porcentajeDescuento) {
      this.porcentajeDescuento = porcentajeDescuento;
   }

   public Double getDescuentoPorcentaje() {
      return descuentoPorcentaje;
   }

   public void setDescuentoPorcentaje(Double descuentoPorcentaje) {
      this.descuentoPorcentaje = descuentoPorcentaje;
   }

   public String getProductosVendidos() {
      return productosVendidos;
   }

   public void setProductosVendidos(String productosVendidos) {
      this.productosVendidos = productosVendidos;
   }

   public String getFechaFormateada() {
      SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-yyyy");
      fechaFormateada = formatter.format(this.fechaVenta);
      return fechaFormateada;
   }

   public void setFechaFormateada(String fechaFormateada) {
      SimpleDateFormat formatter = new SimpleDateFormat("dd-MM-yyyy");
      this.fechaFormateada = formatter.format(fechaFormateada);
   }

}
