/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 * Click nbfs://nbhost/SystemFileSystem/Templates/Classes/Class.java to edit this template
 */
package com.arrebol.taxiservicios.beans.catalog;

import com.arrebol.taxiservicios.beans.Datatable;
import com.arrebol.taxiservicios.beans.TaxiGenericBean;
import com.arrebol.taxiservicios.controller.admin.message.MessageController;
import com.arrebol.taxiservicios.model.catalog.MensajePredefinido;
import com.arrebol.taxiservicios.model.enums.MessageType;
import java.io.Serializable;
import java.util.List;
import java.util.UUID;
import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.view.ViewScoped;
import javax.inject.Named;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.primefaces.event.ReorderEvent;
import org.primefaces.event.RowEditEvent;

/**
 *
 * @author pc one
 */
@Named("mensajePredefinidoBean")
@ViewScoped
public class MensajePredefinidoBean extends TaxiGenericBean implements Serializable, Datatable {

    private static final long serialVersionUID = -2400590341745672164L;
    final Logger logger = LogManager.getLogger(getClass());

    private MessageController messageController;
    private List<MensajePredefinido> listMensajePredefinido;
    private MensajePredefinido selectedMensajePredefinido;
    private String tipoMensaje;
    private String title;    
    private String messageHeader = "Estimado/a [nombre_cliente],\n" +
    "Espero que se encuentre bien. Me pongo en contacto con usted para tratar un asunto importante relacionado con el pago de su abono para el lote [folio_lote] que tiene asignado para la fecha [fecha].\n";
    private String message;

    @PostConstruct
    public void init() {
        logger.info("init");
        try {
            setMessageController(new MessageController());            
            setListMensajePredefinido(getMessageController().fillAllMensajePredefinido(getLoggedUser().getLocation()));
            logger.info("loading mensaje predefinido list");
        } catch (Exception e) {
            logger.error("MensajePredefinidoBean init", e);
        }
    }
    
    public  void clearselected(){
        setSelectedMensajePredefinido(new MensajePredefinido());
        setTitle("");
        setTipoMensaje("");
        setMessage("");
    }
    
    public void updatedForm(){
        setTitle(getSelectedMensajePredefinido().getTitle());
        setTipoMensaje(getSelectedMensajePredefinido().getMessageType().name());
        setMessage(getSelectedMensajePredefinido().getMessage());
    }
    
    public void addMensajePredefinido(){
         logger.info("addMensajePredefinido");
        try {
            setMessageController(new MessageController());    
            MensajePredefinido mensaje = new MensajePredefinido();
            mensaje.setId(UUID.randomUUID().toString());
            mensaje.setTitle(getTitle());
            switch (getTipoMensaje()) {
                case "PAGO_RETARDO":
                    mensaje.setMessageType(MessageType.PAGO_RETARDO);
                    break;
                case "PAGO_CINCO_DIAS":
                    mensaje.setMessageType(MessageType.PAGO_CINCO_DIAS);
                    break;
                case "PAGO_HOY":
                    mensaje.setMessageType(MessageType.PAGO_HOY);
                    break;
                case "PAGO_GENERAL":
                    mensaje.setMessageType(MessageType.PAGO_GENERAL);
                    break;
                default:
                    mensaje.setMessageType(MessageType.OTRO);
                    break;
            }
           mensaje.setMessage(getMessageHeader()+getMessage()+".\n");
           mensaje.setActive(1);
           getMessageController().addMensajePredefinido(mensaje, getLoggedUser(), true);
           
           logger.info("Add mensaje predefinido");
           showMessage(FacesMessage.SEVERITY_INFO,"Se agregó correctamente ","Mensaje predefinido agregado correctamente.");
           getListMensajePredefinido().add(mensaje);
        } catch (Exception e) {
            logger.error("addMensajePredefinido: " + e);
            showMessage(FacesMessage.SEVERITY_FATAL,"Error","Ocurrió un error");
        }
    }

    public void updatedMensajePredefinido(){
         logger.info("updatedMensajePredefinido");
        try {
            getSelectedMensajePredefinido().setTitle(getTitle());
             switch (getTipoMensaje()) {
                case "PAGO_RETARDO":
                    getSelectedMensajePredefinido().setMessageType(MessageType.PAGO_RETARDO);
                    break;
                case "PAGO_CINCO_DIAS":
                    getSelectedMensajePredefinido().setMessageType(MessageType.PAGO_CINCO_DIAS);
                    break;
                case "PAGO_HOY":
                    getSelectedMensajePredefinido().setMessageType(MessageType.PAGO_HOY);
                    break;
                case "PAGO_GENERAL":
                    getSelectedMensajePredefinido().setMessageType(MessageType.PAGO_GENERAL);
                    break;
                default:
                    getSelectedMensajePredefinido().setMessageType(MessageType.OTRO);
                    break;
            }
           getSelectedMensajePredefinido().setMessage(getMessageHeader()+getMessage()+".\n");
           getMessageController().addMensajePredefinido(getSelectedMensajePredefinido(), getLoggedUser(), false);

           logger.info("Updated mensaje predefinido");
           showMessage(FacesMessage.SEVERITY_INFO,"Se actualizó correctamente ","Mensaje predefinido se actualizó correctamente.");
            
        } catch (Exception e) {
            logger.error("updatedMensajePredefinido: " + e);
            showMessage(FacesMessage.SEVERITY_FATAL,"Error","Ocurrió un error");
        }
    }
    
    public void deleteMensajePredefinido(){
         logger.info("deleteMensajePredefinido");
        try {
           getSelectedMensajePredefinido().setActive(0);
           getMessageController().addMensajePredefinido(getSelectedMensajePredefinido(), getLoggedUser(), false);
           logger.info("Delete mensaje predefinido");
           showMessage(FacesMessage.SEVERITY_INFO,"Se elimino correctamente ","Mensaje predefinido se elimino correctamente.");
           getListMensajePredefinido().remove(getSelectedMensajePredefinido());
        } catch (Exception e) {
            logger.error("deleteMensajePredefinido: " + e);
            showMessage(FacesMessage.SEVERITY_FATAL,"Error","Ocurrió un error");
        }
    }
    
    @Override
    public void editRow(RowEditEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void onRowCancel(RowEditEvent event) {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void onRowReorder(ReorderEvent event) {
        showMessage(FacesMessage.SEVERITY_INFO, "Registro Movido", "De columna: " + (event.getFromIndex() + 1) + " a columna: " + (event.getToIndex() + 1));
    }

    @Override
    public void addRow() {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    @Override
    public void deleteRow() {
        throw new UnsupportedOperationException("Not supported yet."); // Generated from nbfs://nbhost/SystemFileSystem/Templates/Classes/Code/GeneratedMethodBody
    }

    public MessageController getMessageController() {
        return messageController;
    }

    public void setMessageController(MessageController messageController) {
        this.messageController = messageController;
    }

    public List<MensajePredefinido> getListMensajePredefinido() {
        return listMensajePredefinido;
    }

    public void setListMensajePredefinido(List<MensajePredefinido> listMensajePredefinido) {
        this.listMensajePredefinido = listMensajePredefinido;
    }

    public MensajePredefinido getSelectedMensajePredefinido() {
        return selectedMensajePredefinido;
    }

    public void setSelectedMensajePredefinido(MensajePredefinido selectedMensajePredefinido) {
        this.selectedMensajePredefinido = selectedMensajePredefinido;
    }

    public String getTipoMensaje() {
        return tipoMensaje;
    }

    public void setTipoMensaje(String tipoMensaje) {
        this.tipoMensaje = tipoMensaje;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public String getMessageHeader() {
        return messageHeader;
    }

    public void setMessageHeader(String messageHeader) {
        this.messageHeader = messageHeader;
    }

}
